# Copyright (c) 2013, Web Notes Technologies Pvt. Ltd. and Contributors
# License: GNU General Public License v3. See license.txt

from __future__ import unicode_literals
import frappe
from frappe import _, throw
from frappe.utils import cint
from frappe.model.controller import DocListController
import frappe.defaults

class PriceList(DocListController):
	def validate(self):
		if not cint(self.doc.buying) and not cint(self.doc.selling):
			throw(_("Price List must be applicable for Buying or Selling"))
				
		if not self.get("valid_for_territories"):
			# if no territory, set default territory
			if frappe.defaults.get_user_default("territory"):
				self.append("valid_for_territories", {
					"doctype": "Applicable Territory",
					"territory": frappe.defaults.get_user_default("territory")
				})
			else:
				# at least one territory
				self.validate_table_has_rows("valid_for_territories")

	def on_update(self):
		self.set_default_if_missing()
		self.update_item_price()

	def set_default_if_missing(self):
		if cint(self.doc.selling):
			if not frappe.db.get_value("Selling Settings", None, "selling_price_list"):
				frappe.set_value("Selling Settings", "Selling Settings", "selling_price_list", self.doc.name)

		elif cint(self.doc.buying):
			if not frappe.db.get_value("Buying Settings", None, "buying_price_list"):
				frappe.set_value("Buying Settings", "Buying Settings", "buying_price_list", self.doc.name)

	def update_item_price(self):
		frappe.db.sql("""update `tabItem Price` set currency=%s, 
			buying=%s, selling=%s, modified=NOW() where price_list=%s""", 
			(self.doc.currency, cint(self.doc.buying), cint(self.doc.selling), self.doc.name))

	def on_trash(self):
		def _update_default_price_list(module):
			b = frappe.bean(module + " Settings")
			price_list_fieldname = module.lower() + "_price_list"

			if self.doc.name == b.doc.fields[price_list_fieldname]:
				b.doc.fields[price_list_fieldname] = None
				b.save()
		
		for module in ["Selling", "Buying"]:
			_update_default_price_list(module)